@@ -1,17 +1,21 @@
-/* $Id: win32_v.cpp 14426 2008-10-01 11:48:57Z rubidium $ */
+/* $Id: win32_v.cpp 18046 2009-11-11 21:45:50Z rubidium $ */
+
+/** @file win32_v.cpp Implementation of the Windows (GDI) video driver. */
 
 #include "../stdafx.h"
 #include "../openttd.h"
 #include "../gfx_func.h"
 #include "../variables.h"
 #include "../win32.h"
+#include "../rev.h"
 #include "../blitter/factory.hpp"
 #include "../network/network.h"
 #include "../core/math_func.hpp"
 #include "../core/random_func.hpp"
+#include "../functions.h"
+#include "../texteff.hpp"
 #include "win32_v.h"
 #include <windows.h>
-#include <tchar.h>
 
 static struct {
 	HWND main_wnd;
@@ -31,7 +35,7 @@ bool _force_full_redraw;
 bool _window_maximize;
 uint _display_hz;
 uint _fullscreen_bpp;
-static uint16 _bck_resolution[2];
+static Dimension _bck_resolution;
 #if !defined(UNICODE)
 uint _codepage;
 #endif
@@ -54,7 +58,7 @@ static void MakePalette()
 
 	}
 	_wnd.gdi_palette = CreatePalette(pal);
-	if (_wnd.gdi_palette == NULL) error("CreatePalette failed!\n");
+	if (_wnd.gdi_palette == NULL) usererror("CreatePalette failed!\n");
 }
 
 static void UpdatePalette(HDC dc, uint start, uint count)
@@ -83,10 +87,10 @@ struct VkMapping {
 
 static const VkMapping _vk_mapping[] = {
 	/* Pageup stuff + up/down */
-	AM(VK_PRIOR,VK_DOWN, WKC_PAGEUP, WKC_DOWN),
+	AM(VK_PRIOR, VK_DOWN, WKC_PAGEUP, WKC_DOWN),
 	/* Map letters & digits */
-	AM('A','Z','A','Z'),
-	AM('0','9','0','9'),
+	AM('A', 'Z', 'A', 'Z'),
+	AM('0', '9', '0', '9'),
 
 	AS(VK_ESCAPE,   WKC_ESC),
 	AS(VK_PAUSE,    WKC_PAUSE),
@@ -144,14 +148,14 @@ static bool AllocateDibSection(int w, int h);
 
 static void ClientSizeChanged(int w, int h)
 {
-	// allocate new dib section of the new size
+	/* allocate new dib section of the new size */
 	if (AllocateDibSection(w, h)) {
-		// mark all palette colors dirty
+		/* mark all palette colors dirty */
 		_pal_first_dirty = 0;
 		_pal_count_dirty = 256;
 		GameSizeChanged();
 
-		// redraw screen
+		/* redraw screen */
 		if (_wnd.running) {
 			_screen.dst_ptr = _wnd.buffer_bits;
 			UpdateWindows();
@@ -160,11 +164,11 @@ static void ClientSizeChanged(int w, int h)
 }
 
 #ifdef _DEBUG
-// Keep this function here..
-// It allows you to redraw the screen from within the MSVC debugger
+/* Keep this function here..
+ * It allows you to redraw the screen from within the MSVC debugger */
 int RedrawScreenDebug()
 {
-	HDC dc,dc2;
+	HDC dc, dc2;
 	static int _fooctr;
 	HBITMAP old_bmp;
 	HPALETTE old_palette;
@@ -216,7 +220,7 @@ static bool MakeWindow(bool full_screen)
 {
 	_fullscreen = full_screen;
 
-	// recreate window?
+	/* recreate window? */
 	if ((full_screen || _wnd.fullscreen) && _wnd.main_wnd) {
 		DestroyWindow(_wnd.main_wnd);
 		_wnd.main_wnd = 0;
@@ -248,7 +252,7 @@ static bool MakeWindow(bool full_screen)
 			return false;  // the request failed
 		}
 	} else if (_wnd.fullscreen) {
-		// restore display?
+		/* restore display? */
 		ChangeDisplaySettings(NULL, 0);
 	}
 #endif
@@ -282,13 +286,12 @@ static bool MakeWindow(bool full_screen)
 			ShowWindow(_wnd.main_wnd, SW_SHOWNORMAL); // remove maximize-flag
 			SetWindowPos(_wnd.main_wnd, 0, x, y, w, h, SWP_NOACTIVATE | SWP_NOOWNERZORDER | SWP_NOZORDER);
 		} else {
-			extern const char _openttd_revision[];
 			TCHAR Windowtitle[50];
 
 			_sntprintf(Windowtitle, lengthof(Windowtitle), _T("OpenTTD %s"), MB_TO_WIDE(_openttd_revision));
 
 			_wnd.main_wnd = CreateWindow(_T("OTTD"), Windowtitle, style, x, y, w, h, 0, 0, GetModuleHandle(NULL), 0);
-			if (_wnd.main_wnd == NULL) error("CreateWindow failed");
+			if (_wnd.main_wnd == NULL) usererror("CreateWindow failed");
 			ShowWindow(_wnd.main_wnd, showstyle);
 		}
 	}
@@ -308,7 +311,7 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 
 		case WM_PAINT: {
 			PAINTSTRUCT ps;
-			HDC dc,dc2;
+			HDC dc, dc2;
 			HBITMAP old_bmp;
 			HPALETTE old_palette;
 
@@ -367,10 +370,7 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			return 0;
 
 		case WM_DESTROY:
-			if (_window_maximize) {
-				_cur_resolution[0] = _bck_resolution[0];
-				_cur_resolution[1] = _bck_resolution[1];
-			}
+			if (_window_maximize) _cur_resolution = _bck_resolution;
 			return 0;
 
 		case WM_LBUTTONDOWN:
@@ -426,8 +426,8 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 				int dx = x - _cursor.pos.x;
 				int dy = y - _cursor.pos.y;
 				if (dx != 0 || dy != 0) {
-					_cursor.delta.x += dx;
-					_cursor.delta.y += dy;
+					_cursor.delta.x = dx;
+					_cursor.delta.y = dy;
 
 					pt.x = _cursor.pos.x;
 					pt.y = _cursor.pos.y;
@@ -436,8 +436,8 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 					SetCursorPos(pt.x, pt.y);
 				}
 			} else {
-				_cursor.delta.x += x - _cursor.pos.x;
-				_cursor.delta.y += y - _cursor.pos.y;
+				_cursor.delta.x = x - _cursor.pos.x;
+				_cursor.delta.y = y - _cursor.pos.y;
 				_cursor.pos.x = x;
 				_cursor.pos.y = y;
 				_cursor.dirty = true;
@@ -500,21 +500,21 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 			return 0;
 		}
 
-		case WM_SYSKEYDOWN: /* user presses F10 or Alt, both activating the title-menu */
+		case WM_SYSKEYDOWN: // user presses F10 or Alt, both activating the title-menu
 			switch (wParam) {
 				case VK_RETURN:
-				case 'F': /* Full Screen on ALT + ENTER/F */
+				case 'F': // Full Screen on ALT + ENTER/F
 					ToggleFullScreen(!_wnd.fullscreen);
 					return 0;
 
-				case VK_MENU: /* Just ALT */
+				case VK_MENU: // Just ALT
 					return 0; // do nothing
 
-				case VK_F10: /* F10, ignore activation of menu */
+				case VK_F10: // F10, ignore activation of menu
 					HandleKeypress(MapWindowsKey(wParam) << 16);
 					return 0;
 
-				default: /* ALT in combination with something else */
+				default: // ALT in combination with something else
 					HandleKeypress(MapWindowsKey(wParam) << 16);
 					break;
 			}
@@ -525,17 +525,14 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 				/* Set maximized flag when we maximize (obviously), but also when we
 				 * switched to fullscreen from a maximized state */
 				_window_maximize = (wParam == SIZE_MAXIMIZED || (_window_maximize && _fullscreen));
-				if (_window_maximize) {
-					_bck_resolution[0] = _cur_resolution[0];
-					_bck_resolution[1] = _cur_resolution[1];
-				}
+				if (_window_maximize) _bck_resolution = _cur_resolution;
 				ClientSizeChanged(LOWORD(lParam), HIWORD(lParam));
 			}
 			return 0;
 
 #if !defined(WINCE)
 		case WM_SIZING: {
-			RECT* r = (RECT*)lParam;
+			RECT *r = (RECT*)lParam;
 			RECT r2;
 			int w, h;
 
@@ -544,8 +541,8 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 
 			w = r->right - r->left - (r2.right - r2.left);
 			h = r->bottom - r->top - (r2.bottom - r2.top);
-			w = Clamp(w, 64, MAX_SCREEN_WIDTH);
-			h = Clamp(h, 64, MAX_SCREEN_HEIGHT);
+			w = max(w, 64);
+			h = max(h, 64);
 			SetRect(&r2, 0, 0, w, h);
 
 			AdjustWindowRect(&r2, GetWindowLong(hwnd, GWL_STYLE), FALSE);
@@ -593,13 +590,13 @@ static LRESULT CALLBACK WndProcGdi(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lP
 		}
 #endif
 
-// needed for wheel
+/* needed for wheel */
 #if !defined(WM_MOUSEWHEEL)
 # define WM_MOUSEWHEEL 0x020A
-#endif  //WM_MOUSEWHEEL
+#endif  /* WM_MOUSEWHEEL */
 #if !defined(GET_WHEEL_DELTA_WPARAM)
 # define GET_WHEEL_DELTA_WPARAM(wparam) ((short)HIWORD(wparam))
-#endif  //GET_WHEEL_DELTA_WPARAM
+#endif  /* GET_WHEEL_DELTA_WPARAM */
 
 		case WM_MOUSEWHEEL: {
 			int delta = GET_WHEEL_DELTA_WPARAM(wParam);
@@ -666,7 +663,7 @@ static void RegisterWndClass()
 		};
 
 		registered = true;
-		if (!RegisterClass(&wnd)) error("RegisterClass failed");
+		if (!RegisterClass(&wnd)) usererror("RegisterClass failed");
 	}
 }
 
@@ -676,10 +673,10 @@ static bool AllocateDibSection(int w, int h)
 	HDC dc;
 	int bpp = BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth();
 
-	w = Clamp(w, 64, MAX_SCREEN_WIDTH);
-	h = Clamp(h, 64, MAX_SCREEN_HEIGHT);
+	w = max(w, 64);
+	h = max(h, 64);
 
-	if (bpp == 0) error("Can't use a blitter that blits 0 bpp for normal visuals");
+	if (bpp == 0) usererror("Can't use a blitter that blits 0 bpp for normal visuals");
 
 	if (w == _screen.width && h == _screen.height)
 		return false;
@@ -702,13 +699,13 @@ static bool AllocateDibSection(int w, int h)
 
 	dc = GetDC(0);
 	_wnd.dib_sect = CreateDIBSection(dc, bi, DIB_RGB_COLORS, (VOID**)&_wnd.buffer_bits, NULL, 0);
-	if (_wnd.dib_sect == NULL) error("CreateDIBSection failed");
+	if (_wnd.dib_sect == NULL) usererror("CreateDIBSection failed");
 	ReleaseDC(0, dc);
 
 	return true;
 }
 
-static const uint16 default_resolutions[][2] = {
+static const Dimension default_resolutions[] = {
 	{  640,  480 },
 	{  800,  600 },
 	{ 1024,  768 },
@@ -726,8 +723,8 @@ static void FindResolutions()
 {
 	uint n = 0;
 #if defined(WINCE)
-	/* EnumDisplaySettingsW is only supported in CE 4.2+ */
-	/* XXX -- One might argue that we assume 4.2+ on every system. Then we can use this function safely */
+	/* EnumDisplaySettingsW is only supported in CE 4.2+
+	 * XXX -- One might argue that we assume 4.2+ on every system. Then we can use this function safely */
 #else
 	uint i;
 	DEVMODEA dm;
@@ -736,12 +733,12 @@ static void FindResolutions()
 	 * Doesn't really matter since we don't pass a string anyways, but still
 	 * a letdown */
 	for (i = 0; EnumDisplaySettingsA(NULL, i, &dm) != 0; i++) {
-		if (dm.dmBitsPerPel == BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth() && IsInsideMM(dm.dmPelsWidth, 640, MAX_SCREEN_WIDTH + 1) &&
-				IsInsideMM(dm.dmPelsHeight, 480, MAX_SCREEN_HEIGHT + 1)) {
+		if (dm.dmBitsPerPel == BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth() &&
+				dm.dmPelsWidth >= 640 && dm.dmPelsHeight >= 480) {
 			uint j;
 
 			for (j = 0; j < n; j++) {
-				if (_resolutions[j][0] == dm.dmPelsWidth && _resolutions[j][1] == dm.dmPelsHeight) break;
+				if (_resolutions[j].width == (int)dm.dmPelsWidth && _resolutions[j].height == (int)dm.dmPelsHeight) break;
 			}
 
 			/* In the previous loop we have checked already existing/added resolutions if
@@ -749,8 +746,8 @@ static void FindResolutions()
 			 * looped all and found none, add the new one to the list. If we have reached the
 			 * maximum amount of resolutions, then quit querying the display */
 			if (j == n) {
-				_resolutions[j][0] = dm.dmPelsWidth;
-				_resolutions[j][1] = dm.dmPelsHeight;
+				_resolutions[j].width  = dm.dmPelsWidth;
+				_resolutions[j].height = dm.dmPelsHeight;
 				if (++n == lengthof(_resolutions)) break;
 			}
 		}
@@ -779,17 +776,17 @@ const char *VideoDriver_Win32::Start(const char * const *parm)
 
 	FindResolutions();
 
-	DEBUG(driver, 2, "Resolution for display: %dx%d", _cur_resolution[0], _cur_resolution[1]);
+	DEBUG(driver, 2, "Resolution for display: %dx%d", _cur_resolution.width, _cur_resolution.height);
 
-	// fullscreen uses those
-	_wnd.width_org = _cur_resolution[0];
-	_wnd.height_org = _cur_resolution[1];
-
-	AllocateDibSection(_cur_resolution[0], _cur_resolution[1]);
-	MarkWholeScreenDirty();
+	/* fullscreen uses those */
+	_wnd.width_org  = _cur_resolution.width;
+	_wnd.height_org = _cur_resolution.height;
 
+	AllocateDibSection(_cur_resolution.width, _cur_resolution.height);
 	MakeWindow(_fullscreen);
 
+	MarkWholeScreenDirty();
+
 	return NULL;
 }
 
@@ -861,7 +858,7 @@ void VideoDriver_Win32::MainLoop()
 			_ctrl_pressed = _wnd.has_focus && GetAsyncKeyState(VK_CONTROL)<0;
 			_shift_pressed = _wnd.has_focus && GetAsyncKeyState(VK_SHIFT)<0;
 
-			// determine which directional keys are down
+			/* determine which directional keys are down */
 			if (_wnd.has_focus) {
 				_dirkeys =
 					(GetAsyncKeyState(VK_LEFT) < 0 ? 1 : 0) +
@@ -875,7 +872,6 @@ void VideoDriver_Win32::MainLoop()
 			if (old_ctrl_pressed != _ctrl_pressed) HandleCtrlChanged();
 
 			GameLoop();
-			_cursor.delta.x = _cursor.delta.y = 0;
 
 			if (_force_full_redraw) MarkWholeScreenDirty();
 
@@ -891,7 +887,7 @@ void VideoDriver_Win32::MainLoop()
 			GdiFlush();
 #endif
 			_screen.dst_ptr = _wnd.buffer_bits;
-			DrawChatMessage();
+			NetworkDrawChatMessage();
 			DrawMouseCursor();
 		}
 	}
